/* SPDX-License-Identifier: (GPL-2.0-only or LGPL-2.1-only)
 *
 * wrapper/types.h
 *
 * Copyright (C) 2017 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
 */

#ifndef _LTTNG_WRAPPER_TYPES_H
#define _LTTNG_WRAPPER_TYPES_H

#define LTTNG_SIZE_MAX	(~(size_t)0)

#endif /* _LTTNG_WRAPPER_TYPES_H */
