/* SPDX-License-Identifier: (GPL-2.0-only or LGPL-2.1-only) */

#ifdef CONFIG_X86_64
#include "x86-32-syscalls_pointers.h"
#endif

#ifdef CONFIG_ARM64
#include "arm-32-syscalls_pointers.h"
#endif

#ifdef CONFIG_PPC64
#include "powerpc-32-syscalls_pointers.h"
#endif
